package net.sweevo.jinks;

import net.minecraft.commands.Commands;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.server.level.ServerPlayer;
import net.neoforged.bus.api.SubscribeEvent;
import net.neoforged.neoforge.event.RegisterCommandsEvent;
import net.neoforged.neoforge.event.entity.player.PlayerEvent;
import net.neoforged.neoforge.event.server.ServerAboutToStartEvent;
import net.neoforged.neoforge.event.server.ServerStoppingEvent;
import net.neoforged.neoforge.event.tick.LevelTickEvent;
import net.neoforged.neoforge.network.PacketDistributor;
import net.neoforged.neoforge.network.registration.NetworkRegistry;
import net.sweevo.jinks.commands.BackupCommand;
import net.sweevo.jinks.commands.PauseCommand;
import net.sweevo.jinks.config.CommonConfig;
import net.sweevo.jinks.config.ServerConfig;
import net.sweevo.jinks.network.Pause;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;

public class EventListener {

    private boolean doBackup;

    @SubscribeEvent
    public void registerCommands(RegisterCommandsEvent event) {
        event.getDispatcher().register(Commands.literal(ExtBackup.MODID)
                .requires(stack -> ServerConfig.commandsCheatsDisabled() || stack.hasPermission(2))
                .then(BackupCommand.register())
                .then(PauseCommand.register()));
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    @SubscribeEvent
    public void onServerStart(ServerAboutToStartEvent event) {
        File script = CommonConfig.getScript();
        if (!script.exists()) {
            try {
                script.getParentFile().mkdirs();
                Files.writeString(script.toPath(), "#!/bin/bash\n# Put your backup script here!\n\nexit 0");
                script.setExecutable(true);
                ExtBackup.LOGGER.info("No backup script was found, a script has been created at {}, please modify it to your liking.", script.getAbsolutePath());
            } catch (IOException | SecurityException e) {
                ExtBackup.LOGGER.error("Backup script does not exist and cannot be created!");
                ExtBackup.LOGGER.error("Disabling ExtBackup!");
                CommonConfig.setEnabled(false);
            }
        }
        ExtBackup.LOGGER.info("Active script: {}", script.getAbsolutePath());
    }

    @SubscribeEvent
    public void onServerTick(LevelTickEvent.Post event) {
        if (event.getLevel() instanceof ServerLevel level && !level.isClientSide
                && level.getGameTime() % 20 == 0 && level == level.getServer().overworld()) {

            if (!level.getServer().getPlayerList().getPlayers().isEmpty() || this.doBackup || CommonConfig.doNoPlayerBackups()) {
                this.doBackup = false;

                boolean done = BackupThread.tryCreateBackup(level.getServer());
                if (done) {
                    ExtBackup.LOGGER.info("Backup done.");
                }
            }
        }
    }

    @SuppressWarnings("null")
    @SubscribeEvent
    public void onPlayerConnect(PlayerEvent.PlayerLoggedInEvent event) {
        ServerPlayer player = (ServerPlayer) event.getEntity();
        //noinspection UnstableApiUsage
        if (CommonConfig.isEnabled() && event.getEntity().getServer() != null && NetworkRegistry.hasChannel(player.connection.connection, null, Pause.ID)) {
            PacketDistributor.sendToPlayer(player, new Pause(BackupData.get(event.getEntity().getServer()).isPaused()));
        }
    }

    @SuppressWarnings("null")
    @SubscribeEvent
    public void onPlayerDisconnect(PlayerEvent.PlayerLoggedOutEvent event) {
        if (event.getEntity() instanceof ServerPlayer player) {
            //noinspection ConstantConditions
            if (player.getServer().getPlayerList().getPlayers().isEmpty()) {
                this.doBackup = true;
            }
        }
    }

    @SubscribeEvent
    public void onServerShutdown(ServerStoppingEvent event) {
        if (CommonConfig.doRunOnShutDown()) {
            boolean done = BackupThread.tryCreateBackup(event.getServer());
            if (done) {
                ExtBackup.LOGGER.info("Backup done.");
            }
        }
    }
}
