package net.sweevo.jinks.config;

import net.neoforged.neoforge.common.ModConfigSpec;

import java.io.File;

import static net.neoforged.fml.loading.FMLPaths.GAMEDIR;


public class CommonConfig {

    public static final ModConfigSpec CONFIG;
    private static final ModConfigSpec.Builder BUILDER = new ModConfigSpec.Builder();
    private static ModConfigSpec.BooleanValue enabled;
    private static ModConfigSpec.IntValue timer;
    private static ModConfigSpec.BooleanValue sendMessages;
    private static ModConfigSpec.BooleanValue noPlayerBackups;
    private static ModConfigSpec.BooleanValue runOnShutdown;
    private static ModConfigSpec.ConfigValue<String> script;
    private static File cachedScript;

    static {
        init(BUILDER);
        CONFIG = BUILDER.build();
    }

    public static void init(ModConfigSpec.Builder builder) {
        enabled = builder.comment("If set false, no backups are being made.")
                .define("enabled", true);
        timer = builder.comment("The time between two backups in minutes", "5 = each 5 minutes", "60 = each hour", "1440 = each day")
                .defineInRange("timer", 10, 1, Short.MAX_VALUE);
        sendMessages = builder.comment("Should message be sent when backup is in the making?")
                .define("sendMessages", true);
        noPlayerBackups = builder.comment("Create backups, even if nobody is online")
                .define("noPlayerBackups", false);
        runOnShutdown = builder.comment("Create backup on server shutdown")
                .define("runOnShutdown", true);
        script = builder.comment("Script location - this script is run on each interval")
                .define("script", "local/extbackup/runbackup.sh");

        builder.push("mod_compat");
        builder.pop();
    }

    public static boolean isEnabled() {
        return enabled.get();
    }

    public static void setEnabled(boolean value) {
        enabled.set(value);
    }

    // converts config value from milliseconds to minutes
    public static int getTimer() {
        return timer.get() * 60 * 1000;
    }

    public static boolean doNoPlayerBackups() {
        return noPlayerBackups.get();
    }

    public static boolean doRunOnShutDown() {
        return runOnShutdown.get();
    }

    public static boolean sendMessages() {
        return sendMessages.get();
    }

    public static File getScript() {
        String scr = script.get();
        if (cachedScript == null) {
            cachedScript = scr.trim().isEmpty() ? new File(GAMEDIR.get() + "local/extbackup/runbackup.sh") : new File(scr.trim());
        }
        return cachedScript;
    }
}
